@@ -2,7 +2,6 @@
 // File:        baseapi.h
 // Description: Simple API for calling tesseract.
 // Author:      Ray Smith
-// Created:     Fri Oct 06 15:35:01 PDT 2006
 //
 // (C) Copyright 2006, Google Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,32 +16,32 @@
 //
 ///////////////////////////////////////////////////////////////////////
 
-#ifndef TESSERACT_API_BASEAPI_H__
-#define TESSERACT_API_BASEAPI_H__
+#ifndef TESSERACT_API_BASEAPI_H_
+#define TESSERACT_API_BASEAPI_H_
 
-#include <stdio.h>
+#include <cstdio>
 // To avoid collision with other typenames include the ABSOLUTE MINIMUM
 // complexity of includes here. Use forward declarations wherever possible
 // and hide includes of complex types in baseapi.cpp.
-#include "platform.h"
+#include "tess_version.h"
 #include "apitypes.h"
+#include "pageiterator.h"
+#include "platform.h"
+#include "publictypes.h"
+#include "resultiterator.h"
+#include "serialis.h"
+#include "tesscallback.h"
 #include "thresholder.h"
 #include "unichar.h"
-#include "tesscallback.h"
-#include "publictypes.h"
-#include "pageiterator.h"
-#include "resultiterator.h"
 
 template <typename T> class GenericVector;
 class PAGE_RES;
 class PAGE_RES_IT;
 class ParagraphModel;
-class BlamerBundle;
+struct BlamerBundle;
 class BLOCK_LIST;
 class DENORM;
-class IMAGE;
 class MATRIX;
-class PBLOB;
 class ROW;
 class STRING;
 class WERD;
@@ -54,42 +53,42 @@ class ETEXT_DESC;
 struct OSResults;
 class TBOX;
 class UNICHARSET;
+class WERD_CHOICE_LIST;
 
-// From oldlist.h
-// TODO(antonova): remove when oldlist is deprecated.
-struct list_rec;
-typedef list_rec *LIST;
-
-#define MAX_NUM_INT_FEATURES 512
 struct INT_FEATURE_STRUCT;
 typedef INT_FEATURE_STRUCT *INT_FEATURE;
-typedef INT_FEATURE_STRUCT INT_FEATURE_ARRAY[MAX_NUM_INT_FEATURES];
 struct TBLOB;
 
 namespace tesseract {
 
-class CubeRecoContext;
 class Dawg;
 class Dict;
 class EquationDetect;
+class PageIterator;
 class LTRResultIterator;
+class ResultIterator;
 class MutableIterator;
+class TessResultRenderer;
 class Tesseract;
 class Trie;
 class Wordrec;
 
 typedef int (Dict::*DictFunc)(void* void_dawg_args,
+                              const UNICHARSET& unicharset,
                               UNICHAR_ID unichar_id, bool word_end) const;
 typedef double (Dict::*ProbabilityInContextFunc)(const char* lang,
                                                  const char* context,
                                                  int context_bytes,
                                                  const char* character,
                                                  int character_bytes);
+typedef float (Dict::*ParamsModelClassifyFunc)(
+    const char *lang, void *path);
 typedef void (Wordrec::*FillLatticeFunc)(const MATRIX &ratings,
-                                         const LIST &best_choices,
+                                         const WERD_CHOICE_LIST &best_choices,
                                          const UNICHARSET &unicharset,
                                          BlamerBundle *blamer_bundle);
-typedef TessCallback3<const UNICHARSET &, int, PAGE_RES *> TruthCallback;
+typedef TessCallback4<const UNICHARSET &, int, PageIterator *, Pix *>
+    TruthCallback;
 
 /**
  * Base class for all tesseract APIs.
@@ -110,10 +109,38 @@ class TESS_API TessBaseAPI {
   static const char* Version();
 
   /**
-   * Set the name of the input file. Needed only for training and
-   * reading a UNLV zone file.
+   * If compiled with OpenCL AND an available OpenCL
+   * device is deemed faster than serial code, then
+   * "device" is populated with the cl_device_id
+   * and returns sizeof(cl_device_id)
+   * otherwise *device=nullptr and returns 0.
+   */
+  static size_t getOpenCLDevice(void **device);
+
+  /**
+   * Writes the thresholded image to stderr as a PBM file on receipt of a
+   * SIGSEGV, SIGFPE, or SIGBUS signal. (Linux/Unix only).
+   */
+  static void CatchSignals();
+
+  /**
+   * Set the name of the input file. Needed for training and
+   * reading a UNLV zone file, and for searchable PDF output.
    */
   void SetInputName(const char* name);
+  /**
+   * These functions are required for searchable PDF output.
+   * We need our hands on the input file so that we can include
+   * it in the PDF without transcoding. If that is not possible,
+   * we need the original image. Finally, resolution metadata
+   * is stored in the PDF so we need that as well.
+   */
+  const char* GetInputName();
+  // Takes ownership of the input pix.
+  void SetInputImage(Pix *pix);
+  Pix* GetInputImage();
+  int GetSourceYResolution();
+  const char* GetDatapath();
 
   /** Set the name of the bonus output files. Needed only for debugging. */
   void SetOutputName(const char* name);
@@ -127,8 +154,6 @@ class TESS_API TessBaseAPI {
    * Or SetVariable("classify_bln_numeric_mode", "1"); to set numeric-only mode.
    * SetVariable may be used before Init, but settings will revert to
    * defaults on End().
-   * TODO(rays) Add a command-line option to dump the parameters to stdout
-   * and add a pointer to it in the FAQ
    *
    * Note: Must be called after Init(). Only works for non-init variables
    * (init variables should be passed to Init()).
@@ -173,7 +198,7 @@ class TESS_API TessBaseAPI {
    *
    * The datapath must be the name of the parent directory of tessdata and
    * must end in / . Any name after the last / will be stripped.
-   * The language is (usually) an ISO 639-3 string or NULL will default to eng.
+   * The language is (usually) an ISO 639-3 string or nullptr will default to eng.
    * It is entirely safe (and eventually will be efficient too) to call
    * Init multiple times on the same instance to change language, or just
    * to reset the classifier.
@@ -204,11 +229,18 @@ class TESS_API TessBaseAPI {
            const GenericVector<STRING> *vars_values,
            bool set_only_non_debug_params);
   int Init(const char* datapath, const char* language, OcrEngineMode oem) {
-    return Init(datapath, language, oem, NULL, 0, NULL, NULL, false);
+    return Init(datapath, language, oem, nullptr, 0, nullptr, nullptr, false);
   }
   int Init(const char* datapath, const char* language) {
-    return Init(datapath, language, OEM_DEFAULT, NULL, 0, NULL, NULL, false);
+    return Init(datapath, language, OEM_DEFAULT, nullptr, 0, nullptr, nullptr, false);
   }
+  // In-memory version reads the traineddata file directly from the given
+  // data[data_size] array, and/or reads data via a FileReader.
+  int Init(const char* data, int data_size, const char* language,
+           OcrEngineMode mode, char** configs, int configs_size,
+           const GenericVector<STRING>* vars_vec,
+           const GenericVector<STRING>* vars_values,
+           bool set_only_non_debug_params, FileReader reader);
 
   /**
    * Returns the languages string used in the last valid initialization.
@@ -228,7 +260,7 @@ class TESS_API TessBaseAPI {
   void GetLoadedLanguagesAsVector(GenericVector<STRING>* langs) const;
 
   /**
-   * Returns the available languages in the vector of STRINGs.
+   * Returns the available languages in the sorted vector of STRINGs.
    */
   void GetAvailableLanguagesAsVector(GenericVector<STRING>* langs) const;
 
@@ -303,9 +335,7 @@ class TESS_API TessBaseAPI {
 
   /**
    * Provide an image for Tesseract to recognize. Format is as
-   * TesseractRect above. Does not copy the image buffer, or take
-   * ownership. The source image may be destroyed after Recognize is called,
-   * either explicitly or implicitly via one of the Get*Text functions.
+   * TesseractRect above. Copies the image buffer and converts to Pix.
    * SetImage clears all recognition results, and sets the rectangle to the
    * full image, so it may be followed immediately by a GetUTF8Text, and it
    * will automatically perform recognition.
@@ -315,15 +345,13 @@ class TESS_API TessBaseAPI {
 
   /**
    * Provide an image for Tesseract to recognize. As with SetImage above,
-   * Tesseract doesn't take a copy or ownership or pixDestroy the image, so
-   * it must persist until after Recognize.
+   * Tesseract takes its own copy of the image, so it need not persist until
+   * after Recognize.
    * Pix vs raw, which to use?
-   * Use Pix where possible. A future version of Tesseract may choose to use Pix
-   * as its internal representation and discard IMAGE altogether.
-   * Because of that, an implementation that sources and targets Pix may end up
-   * with less copies than an implementation that does not.
+   * Use Pix where possible. Tesseract uses Pix as its internal representation
+   * and it is therefore more efficient to provide a Pix directly.
    */
-  void SetImage(const Pix* pix);
+  void SetImage(Pix* pix);
 
   /**
    * Set the resolution of the source image in pixels per inch so font size
@@ -346,8 +374,7 @@ class TESS_API TessBaseAPI {
    * delete it when it it is replaced or the API is destructed.
    */
   void SetThresholder(ImageThresholder* thresholder) {
-    if (thresholder_ != NULL)
-      delete thresholder_;
+    delete thresholder_;
     thresholder_ = thresholder;
     ClearResults();
   }
@@ -370,17 +397,28 @@ class TESS_API TessBaseAPI {
    * Get the textlines as a leptonica-style
    * Boxa, Pixa pair, in reading order.
    * Can be called before or after Recognize.
-   * If blockids is not NULL, the block-id of each line is also returned
-   * as an array of one element per line. delete [] after use.
+   * If raw_image is true, then extract from the original image instead of the
+   * thresholded image and pad by raw_padding pixels.
+   * If blockids is not nullptr, the block-id of each line is also returned as an
+   * array of one element per line. delete [] after use.
+   * If paraids is not nullptr, the paragraph-id of each line within its block is
+   * also returned as an array of one element per line. delete [] after use.
    */
-  Boxa* GetTextlines(Pixa** pixa, int** blockids);
+  Boxa* GetTextlines(const bool raw_image, const int raw_padding,
+                     Pixa** pixa, int** blockids, int** paraids);
+  /*
+     Helper method to extract from the thresholded image. (most common usage)
+  */
+  Boxa* GetTextlines(Pixa** pixa, int** blockids) {
+    return GetTextlines(false, 0, pixa, blockids, nullptr);
+  }
 
   /**
    * Get textlines and strips of image regions as a leptonica-style Boxa, Pixa
    * pair, in reading order. Enables downstream handling of non-rectangular
    * regions.
    * Can be called before or after Recognize.
-   * If blockids is not NULL, the block-id of each line is also returned as an
+   * If blockids is not nullptr, the block-id of each line is also returned as an
    * array of one element per line. delete [] after use.
    */
   Boxa* GetStrips(Pixa** pixa, int** blockids);
@@ -406,13 +444,25 @@ class TESS_API TessBaseAPI {
    * Get the given level kind of components (block, textline, word etc.) as a
    * leptonica-style Boxa, Pixa pair, in reading order.
    * Can be called before or after Recognize.
-   * If blockids is not NULL, the block-id of each component is also returned
+   * If blockids is not nullptr, the block-id of each component is also returned
    * as an array of one element per component. delete [] after use.
+   * If blockids is not nullptr, the paragraph-id of each component with its block
+   * is also returned as an array of one element per component. delete [] after
+   * use.
+   * If raw_image is true, then portions of the original image are extracted
+   * instead of the thresholded image and padded with raw_padding.
    * If text_only is true, then only text components are returned.
    */
-  Boxa* GetComponentImages(PageIteratorLevel level,
-                           bool text_only,
-                           Pixa** pixa, int** blockids);
+  Boxa* GetComponentImages(const PageIteratorLevel level,
+                           const bool text_only, const bool raw_image,
+                           const int raw_padding,
+                           Pixa** pixa, int** blockids, int** paraids);
+  // Helper function to get binary images with no padding (most common usage).
+  Boxa* GetComponentImages(const PageIteratorLevel level,
+                           const bool text_only,
+                           Pixa** pixa, int** blockids) {
+    return GetComponentImages(level, text_only, false, 0, pixa, blockids, nullptr);
+  }
 
   /**
    * Returns the scale factor of the thresholded image that would be returned by
@@ -422,18 +472,15 @@ class TESS_API TessBaseAPI {
    */
   int GetThresholdedImageScaleFactor() const;
 
-  /**
-   * Dump the internal binary image to a PGM file.
-   * @deprecated Use GetThresholdedImage and write the image using pixWrite
-   * instead if possible.
-   */
-  void DumpPGM(const char* filename);
-
   /**
    * Runs page layout analysis in the mode set by SetPageSegMode.
    * May optionally be called prior to Recognize to get access to just
    * the page layout results. Returns an iterator to the results.
-   * Returns NULL on error.
+   * If merge_similar_words is true, words are combined where suitable for use
+   * with a line recognizer. Use if you want to use AnalyseLayout to find the
+   * textlines, and then want to process textline fragments with an external
+   * line recognizer.
+   * Returns nullptr on error or an empty page.
    * The returned iterator must be deleted after use.
    * WARNING! This class points to data held within the TessBaseAPI class, and
    * therefore can only be used while the TessBaseAPI class still exists and
@@ -441,6 +488,7 @@ class TESS_API TessBaseAPI {
    * DetectOS, or anything else that changes the internal PAGE_RES.
    */
   PageIterator* AnalyseLayout();
+  PageIterator* AnalyseLayout(bool merge_similar_words);
 
   /**
    * Recognize the image from SetAndThresholdImage, generating Tesseract
@@ -455,43 +503,51 @@ class TESS_API TessBaseAPI {
    * Recognize() or TesseractRect(). (Recognize is called implicitly if needed.)
    */
 
+  #ifndef DISABLED_LEGACY_ENGINE
   /** Variant on Recognize used for testing chopper. */
   int RecognizeForChopTest(ETEXT_DESC* monitor);
+  #endif
 
   /**
-   * Recognizes all the pages in the named file, as a multi-page tiff or
-   * list of filenames, or single image, and gets the appropriate kind of text
-   * according to parameters: tessedit_create_boxfile,
-   * tessedit_make_boxes_from_boxes, tessedit_write_unlv, tessedit_create_hocr.
-   * Calls ProcessPage on each page in the input file, which may be a
-   * multi-page tiff, single-page other file format, or a plain text list of
-   * images to read. If tessedit_page_number is non-negative, processing begins
-   * at that page of a multi-page tiff file, or filelist.
-   * The text is returned in text_out. Returns false on error.
-   * If non-zero timeout_millisec terminates processing after the timeout on
-   * a single page.
-   * If non-NULL and non-empty, and some page fails for some reason,
-   * the page is reprocessed with the retry_config config file. Useful
-   * for interactively debugging a bad page.
+   * Turns images into symbolic text.
+   *
+   * filename can point to a single image, a multi-page TIFF,
+   * or a plain text list of image filenames.
+   *
+   * retry_config is useful for debugging. If not nullptr, you can fall
+   * back to an alternate configuration if a page fails for some
+   * reason.
+   *
+   * timeout_millisec terminates processing if any single page
+   * takes too long. Set to 0 for unlimited time.
+   *
+   * renderer is responible for creating the output. For example,
+   * use the TessTextRenderer if you want plaintext output, or
+   * the TessPDFRender to produce searchable PDF.
+   *
+   * If tessedit_page_number is non-negative, will only process that
+   * single page. Works for multi-page tiff file, or filelist.
+   *
+   * Returns true if successful, false on error.
    */
-  bool ProcessPages(const char* filename,
-                    const char* retry_config, int timeout_millisec,
-                    STRING* text_out);
+  bool ProcessPages(const char* filename, const char* retry_config,
+                    int timeout_millisec, TessResultRenderer* renderer);
+  // Does the real work of ProcessPages.
+  bool ProcessPagesInternal(const char* filename, const char* retry_config,
+                            int timeout_millisec, TessResultRenderer* renderer);
 
   /**
-   * Recognizes a single page for ProcessPages, appending the text to text_out.
-   * The pix is the image processed - filename and page_index are metadata
-   * used by side-effect processes, such as reading a box file or formatting
-   * as hOCR.
-   * If non-zero timeout_millisec terminates processing after the timeout.
-   * If non-NULL and non-empty, and some page fails for some reason,
-   * the page is reprocessed with the retry_config config file. Useful
-   * for interactively debugging a bad page.
-   * The text is returned in text_out. Returns false on error.
+   * Turn a single image into symbolic text.
+   *
+   * The pix is the image processed. filename and page_index are
+   * metadata used by side-effect processes, such as reading a box
+   * file or formatting as hOCR.
+   *
+   * See ProcessPages for desciptions of other parameters.
    */
   bool ProcessPage(Pix* pix, int page_index, const char* filename,
                    const char* retry_config, int timeout_millisec,
-                   STRING* text_out);
+                   TessResultRenderer* renderer);
 
   /**
    * Get a reading-order iterator to the results of LayoutAnalysis and/or
@@ -523,22 +579,63 @@ class TESS_API TessBaseAPI {
    * Make a HTML-formatted string with hOCR markup from the internal
    * data structures.
    * page_number is 0-based but will appear in the output as 1-based.
+   * monitor can be used to
+   *  cancel the recognition
+   *  receive progress callbacks
+   * Returned string must be freed with the delete [] operator.
+   */
+  char* GetHOCRText(ETEXT_DESC* monitor, int page_number);
+
+  /**
+   * Make a HTML-formatted string with hOCR markup from the internal
+   * data structures.
+   * page_number is 0-based but will appear in the output as 1-based.
+   * Returned string must be freed with the delete [] operator.
    */
   char* GetHOCRText(int page_number);
+
+  /**
+   * Make a TSV-formatted string from the internal data structures.
+   * page_number is 0-based but will appear in the output as 1-based.
+   * Returned string must be freed with the delete [] operator.
+   */
+  char* GetTSVText(int page_number);
+
   /**
    * The recognized text is returned as a char* which is coded in the same
-   * format as a box file used in training. Returned string must be freed with
-   * the delete [] operator.
+   * format as a box file used in training.
    * Constructs coordinates in the original image - not just the rectangle.
    * page_number is a 0-based page index that will appear in the box file.
+   * Returned string must be freed with the delete [] operator.
    */
   char* GetBoxText(int page_number);
+
   /**
    * The recognized text is returned as a char* which is coded
-   * as UNLV format Latin-1 with specific reject and suspect codes
-   * and must be freed with the delete [] operator.
+   * as UNLV format Latin-1 with specific reject and suspect codes.
+   * Returned string must be freed with the delete [] operator.
    */
   char* GetUNLVText();
+
+  /**
+   * Detect the orientation of the input image and apparent script (alphabet).
+   * orient_deg is the detected clockwise rotation of the input image in degrees
+   * (0, 90, 180, 270)
+   * orient_conf is the confidence (15.0 is reasonably confident)
+   * script_name is an ASCII string, the name of the script, e.g. "Latin"
+   * script_conf is confidence level in the script
+   * Returns true on success and writes values to each parameter as an output
+   */
+  bool DetectOrientationScript(int* orient_deg, float* orient_conf,
+                               const char** script_name, float* script_conf);
+
+  /**
+   * The recognized text is returned as a char* which is coded
+   * as UTF8 and must be freed with the delete [] operator.
+   * page_number is a 0-based page index that will appear in the osd file.
+   */
+  char* GetOsdText(int page_number);
+
   /** Returns the (average) confidence value between 0 and 100. */
   int MeanTextConf();
   /**
@@ -549,6 +646,7 @@ class TESS_API TessBaseAPI {
    */
   int* AllWordConfidences();
 
+#ifndef DISABLED_LEGACY_ENGINE
   /**
    * Applies the given word to the adaptive classifier if possible.
    * The word must be SPACE-DELIMITED UTF-8 - l i k e t h i s , so it can
@@ -560,6 +658,7 @@ class TESS_API TessBaseAPI {
    * Returns false if adaption was not possible for some reason.
    */
   bool AdaptToWordStr(PageSegMode mode, const char* wordstr);
+#endif  //  ndef DISABLED_LEGACY_ENGINE
 
   /**
    * Free up recognition results and any stored image data, without actually
@@ -577,6 +676,15 @@ class TESS_API TessBaseAPI {
    */
   void End();
 
+  /**
+   * Clear any library-level memory caches.
+   * There are a variety of expensive-to-load constant data structures (mostly
+   * language dictionaries) that are cached globally -- surviving the Init()
+   * and End() of individual TessBaseAPI's.  This function allows the clearing
+   * of these caches.
+   **/
+  static void ClearPersistentCache();
+
   /**
    * Check whether a word is valid according to Tesseract's language model
    * @return 0 if the word is invalid, non-zero if valid.
@@ -584,6 +692,9 @@ class TESS_API TessBaseAPI {
    * in a separate API at some future time.
    */
   int IsValidWord(const char *word);
+  // Returns true if utf8_character is defined in the UniCharset.
+  bool IsValidCharacter(const char *utf8_character);
+
 
   bool GetTextDirection(int* out_offset, float* out_slope);
 
@@ -595,45 +706,34 @@ class TESS_API TessBaseAPI {
    */
   void SetProbabilityInContextFunc(ProbabilityInContextFunc f);
 
-  /** Sets Wordrec::fill_lattice_ function to point to the given function. */
-  void SetFillLatticeFunc(FillLatticeFunc f);
-
   /**
    * Estimates the Orientation And Script of the image.
    * @return true if the image was processed successfully.
    */
   bool DetectOS(OSResults*);
 
-  /** This method returns the features associated with the input image. */
-  void GetFeaturesForBlob(TBLOB* blob, const DENORM& denorm,
-                          INT_FEATURE_ARRAY int_features,
-                          int* num_features, int* FeatureOutlineIndex);
+  /**
+   * Return text orientation of each block as determined by an earlier run
+   * of layout analysis.
+   */
+  void GetBlockTextOrientations(int** block_orientation,
+                                bool** vertical_writing);
+
+
+  #ifndef DISABLED_LEGACY_ENGINE
+
+  /** Sets Wordrec::fill_lattice_ function to point to the given function. */
+  void SetFillLatticeFunc(FillLatticeFunc f);
+
+  /** Find lines from the image making the BLOCK_LIST. */
+  BLOCK_LIST* FindLinesCreateBlockList();
 
   /**
-   * This method returns the row to which a box of specified dimensions would
-   * belong. If no good match is found, it returns NULL.
+   * Delete a block list.
+   * This is to keep BLOCK_LIST pointer opaque
+   * and let go of including the other headers.
    */
-  static ROW* FindRowForBox(BLOCK_LIST* blocks, int left, int top,
-                            int right, int bottom);
-
-  /**
-   * Method to run adaptive classifier on a blob.
-   * It returns at max num_max_matches results.
-   */
-  void RunAdaptiveClassifier(TBLOB* blob, const DENORM& denorm,
-                             int num_max_matches,
-                             int* unichar_ids,
-                             float* ratings,
-                             int* num_matches_returned);
-
-  /** This method returns the string form of the specified unichar. */
-  const char* GetUnichar(int unichar_id);
-
-  /** Return the pointer to the i-th dawg loaded into tesseract_ object. */
-  const Dawg *GetDawg(int i) const;
-
-  /** Return the number of dawgs loaded into tesseract_ object. */
-  int NumDawgs() const;
+  static void DeleteBlockList(BLOCK_LIST* block_list);
 
   /** Returns a ROW object created from the input row specification. */
   static ROW *MakeTessOCRRow(float baseline, float xheight,
@@ -647,40 +747,46 @@ class TESS_API TessBaseAPI {
    * for normalization. The denorm is an optional parameter in which the
    * normalization-antidote is returned.
    */
-  static void NormalizeTBLOB(TBLOB *tblob, ROW *row,
-                             bool numeric_mode, DENORM *denorm);
+  static void NormalizeTBLOB(TBLOB *tblob, ROW *row, bool numeric_mode);
 
-  Tesseract* const tesseract() const {
-    return tesseract_;
-  }
-  
-  OcrEngineMode const oem() const {
-    return last_oem_requested_;
-  }
+  /** This method returns the features associated with the input image. */
+  void GetFeaturesForBlob(TBLOB* blob, INT_FEATURE_STRUCT* int_features,
+                          int* num_features, int* feature_outline_index);
+
+  /**
+   * This method returns the row to which a box of specified dimensions would
+   * belong. If no good match is found, it returns nullptr.
+   */
+  static ROW* FindRowForBox(BLOCK_LIST* blocks, int left, int top,
+                            int right, int bottom);
+
+  /**
+   * Method to run adaptive classifier on a blob.
+   * It returns at max num_max_matches results.
+   */
+  void RunAdaptiveClassifier(TBLOB* blob,
+                             int num_max_matches,
+                             int* unichar_ids,
+                             float* ratings,
+                             int* num_matches_returned);
+#endif  // ndef DISABLED_LEGACY_ENGINE
+
+  /** This method returns the string form of the specified unichar. */
+  const char* GetUnichar(int unichar_id);
+
+  /** Return the pointer to the i-th dawg loaded into tesseract_ object. */
+  const Dawg *GetDawg(int i) const;
+
+  /** Return the number of dawgs loaded into tesseract_ object. */
+  int NumDawgs() const;
+
+  Tesseract* tesseract() const { return tesseract_; }
+
+  OcrEngineMode oem() const { return last_oem_requested_; }
 
   void InitTruthCallback(TruthCallback *cb) { truth_cb_ = cb; }
 
-  /** Return a pointer to underlying CubeRecoContext object if present. */
-  CubeRecoContext *GetCubeRecoContext() const;
-
   void set_min_orientation_margin(double margin);
-
-  /**
-   * Return text orientation of each block as determined by an earlier run
-   * of layout analysis.
-   */
-  void GetBlockTextOrientations(int** block_orientation,
-                                bool** vertical_writing);
-
-  /** Find lines from the image making the BLOCK_LIST. */
-  BLOCK_LIST* FindLinesCreateBlockList();
-
-  /**
-   * Delete a block list.
-   * This is to keep BLOCK_LIST pointer opaque
-   * and let go of including the other headers.
-   */
-  static void DeleteBlockList(BLOCK_LIST* block_list);
  /* @} */
 
  protected:
@@ -689,10 +795,10 @@ class TESS_API TessBaseAPI {
   TESS_LOCAL bool InternalSetImage();
 
   /**
-   * Run the thresholder to make the thresholded image. If pix is not NULL,
+   * Run the thresholder to make the thresholded image. If pix is not nullptr,
    * the source is thresholded to pix instead of the internal IMAGE.
    */
-  TESS_LOCAL virtual void Threshold(Pix** pix);
+  TESS_LOCAL virtual bool Threshold(Pix** pix);
 
   /**
    * Find lines from the image making the BLOCK_LIST.
@@ -701,7 +807,7 @@ class TESS_API TessBaseAPI {
   TESS_LOCAL int FindLines();
 
   /** Delete the pageres and block list ready for a new page. */
-  TESS_LOCAL void ClearResults();
+  void ClearResults();
 
   /**
    * Return an LTR Result Iterator -- used only for training, as we really want
@@ -718,6 +824,11 @@ class TESS_API TessBaseAPI {
    */
   TESS_LOCAL int TextLength(int* blob_count);
 
+  //// paragraphs.cpp ////////////////////////////////////////////////////
+  TESS_LOCAL void DetectParagraphs(bool after_text_recognition);
+
+  #ifndef DISABLED_LEGACY_ENGINE
+
   /** @defgroup ocropusAddOns ocropus add-ons */
   /* @{ */
 
@@ -734,10 +845,9 @@ class TESS_API TessBaseAPI {
 
   /** Recognize text doing one pass only, using settings for a given pass. */
   TESS_LOCAL PAGE_RES* RecognitionPass1(BLOCK_LIST* block_list);
-  TESS_LOCAL PAGE_RES* RecognitionPass2(BLOCK_LIST* block_list, PAGE_RES* pass1_result);
 
-  //// paragraphs.cpp ////////////////////////////////////////////////////
-  TESS_LOCAL void DetectParagraphs(bool after_text_recognition);
+  TESS_LOCAL PAGE_RES* RecognitionPass2(BLOCK_LIST* block_list,
+                                        PAGE_RES* pass1_result);
 
   /**
    * Extract the OCR results, costs (penalty points for uncertainty),
@@ -752,15 +862,15 @@ class TESS_API TessBaseAPI {
                                     int** y1,
                                     PAGE_RES* page_res);
 
-  TESS_LOCAL const PAGE_RES* GetPageRes() const {
-    return page_res_;
-  };
+  TESS_LOCAL const PAGE_RES* GetPageRes() const { return page_res_; }
   /* @} */
+#endif  // ndef DISABLED_LEGACY_ENGINE
 
  protected:
   Tesseract*        tesseract_;       ///< The underlying data object.
   Tesseract*        osd_tesseract_;   ///< For orientation & script detection.
   EquationDetect*   equ_detect_;      ///<The equation detector.
+  FileReader reader_;                 ///< Reads files from any filesystem.
   ImageThresholder* thresholder_;     ///< Image thresholding module.
   GenericVector<ParagraphModel *>* paragraph_models_;
   BLOCK_LIST*       block_list_;      ///< The page layout.
@@ -786,8 +896,31 @@ class TESS_API TessBaseAPI {
   int image_height_;
   /* @} */
 
-};
+ private:
+  // A list of image filenames gets special consideration
+  bool ProcessPagesFileList(FILE *fp,
+                            STRING *buf,
+                            const char* retry_config, int timeout_millisec,
+                            TessResultRenderer* renderer,
+                            int tessedit_page_number);
+  // TIFF supports multipage so gets special consideration.
+  bool ProcessPagesMultipageTiff(const unsigned char *data,
+                                 size_t size,
+                                 const char* filename,
+                                 const char* retry_config,
+                                 int timeout_millisec,
+                                 TessResultRenderer* renderer,
+                                 int tessedit_page_number);
+  // There's currently no way to pass a document title from the
+  // Tesseract command line, and we have multiple places that choose
+  // to set the title to an empty string. Using a single named
+  // variable will hopefully reduce confusion if the situation changes
+  // in the future.
+  const char *unknown_title_ = "";
+};  // class TessBaseAPI.
 
+/** Escape a char string - remove &<>"' with HTML codes. */
+STRING HOcrEscape(const char* text);
 }  // namespace tesseract.
 
-#endif  // TESSERACT_API_BASEAPI_H__
+#endif  // TESSERACT_API_BASEAPI_H_
