@@ -17,26 +17,27 @@
 //
 ///////////////////////////////////////////////////////////////////////
 
-#ifndef TESSERACT_CCUTIL_PLATFORM_H__
-#define TESSERACT_CCUTIL_PLATFORM_H__
+#ifndef TESSERACT_CCUTIL_PLATFORM_H_
+#define TESSERACT_CCUTIL_PLATFORM_H_
 
 #define DLLSYM
-#ifdef __MSW32__
+#ifdef _WIN32
+#ifndef NOMINMAX
+#define NOMINMAX
+#endif /* NOMINMAX */
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
 #ifdef __GNUC__
 #define ultoa _ultoa
-typedef struct _BLOB {
-  unsigned int cbSize;
-  char *pBlobData;
-} BLOB, *LPBLOB;
 #endif  /* __GNUC__ */
 #define SIGNED
-#define snprintf _snprintf
-#if (_MSC_VER <= 1400)
-#define vsnprintf _vsnprintf
-#endif /* __MSW32__ */
 #else
-#define __UNIX__
+#ifdef __cplusplus
+#include <climits>
+#else /* C compiler*/
 #include <limits.h>
+#endif /* __cplusplus */
 #ifndef PATH_MAX
 #define MAX_PATH 4096
 #else
@@ -45,4 +46,34 @@ typedef struct _BLOB {
 #define SIGNED signed
 #endif
 
-#endif  // TESSERACT_CCUTIL_PLATFORM_H__
+#if defined(_WIN32) || defined(__CYGWIN__)
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+#endif
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+    #if defined(TESS_EXPORTS)
+       #define TESS_API __declspec(dllexport)
+    #elif defined(TESS_IMPORTS)
+       #define TESS_API __declspec(dllimport)
+    #else
+       #define TESS_API
+    #endif
+    #define TESS_LOCAL
+#else
+    #if __GNUC__ >= 4
+      #if defined(TESS_EXPORTS) || defined(TESS_IMPORTS)
+          #define TESS_API  __attribute__ ((visibility ("default")))
+          #define TESS_LOCAL  __attribute__ ((visibility ("hidden")))
+      #else
+          #define TESS_API
+          #define TESS_LOCAL
+      #endif
+    #else
+      #define TESS_API
+      #define TESS_LOCAL
+    #endif
+#endif
+
+#endif  // TESSERACT_CCUTIL_PLATFORM_H_
